"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fs_1 = require("fs");
var csv_stringify_1 = __importDefault(require("csv-stringify"));
var TranslationParser = /** @class */ (function () {
    function TranslationParser(dir) {
        this.dir = dir;
    }
    TranslationParser.prototype.parse = function () {
        if (!this.dir || typeof this.dir !== 'string') {
            throw new Error('Directory required');
        }
        var translations = this.walkDir(this.dir);
        var uniqueTranslations = translations.filter(function (value, index, arr) { return arr.indexOf(value) === index; });
        return uniqueTranslations;
    };
    TranslationParser.prototype.getCSVStream = function (translations, header) {
        if (!Array.isArray(translations)) {
            throw new Error('Translations have to be an array');
        }
        if (!Array.isArray(header)) {
            throw new Error('Header has to be an array');
        }
        if (translations[0].length !== header.length) {
            throw new Error('Content and header length differ');
        }
        translations.unshift(header);
        return csv_stringify_1.default(translations, { quoted: true });
    };
    TranslationParser.prototype.walkDir = function (dir, arr, deep) {
        if (arr === void 0) { arr = []; }
        if (deep === void 0) { deep = 50; }
        if (!fs_1.existsSync(dir)) {
            return arr;
        }
        if (deep <= 0) {
            return arr;
        }
        var directoryItems = fs_1.readdirSync(dir);
        for (var _i = 0, directoryItems_1 = directoryItems; _i < directoryItems_1.length; _i++) {
            var item = directoryItems_1[_i];
            var currentDir = dir + "/" + item;
            if (item.charAt(0) === '.' || item.endsWith('.d.ts')) {
                continue;
            }
            var stat = fs_1.lstatSync(currentDir);
            if (stat.isFile()) {
                var translations = this.findTranslationStrings(currentDir);
                if (translations.length) {
                    arr.push.apply(arr, translations);
                }
            }
            if (stat.isDirectory()) {
                this.walkDir(currentDir, arr, deep--);
            }
        }
        return arr;
    };
    TranslationParser.prototype.findTranslationStrings = function (dir) {
        if (!fs_1.existsSync(dir)) {
            return [];
        }
        var fileContent = fs_1.readFileSync(dir);
        var matches = this.matchAll(fileContent.toString(), TranslationParser.TRANSLATION_REG_EXP);
        if (!matches.length) {
            return [];
        }
        var sanitizedMatches = matches.map(function (match) { return match.filter(function (e) { return !!e; }); });
        return sanitizedMatches.reduce(function (translationStringsArray, match) {
            var translationText = match.find(function (e) { return !TranslationParser.TRANSLATION_TEXT_REG_EXP.test(e); });
            if (translationText) {
                translationStringsArray.push(translationText);
            }
            return translationStringsArray;
        }, []);
    };
    TranslationParser.prototype.matchAll = function (str, regexp) {
        var matches = [];
        var iterations = 0;
        var match = regexp.exec(str);
        while (match !== null && iterations < 1000) {
            matches.push(match);
            match = regexp.exec(str);
            iterations++;
        }
        return matches;
    };
    TranslationParser.TRANSLATION_REG_EXP = /(?:\$T|translate|core\.T)\(\n*\s*(?:(?:'(.*?)')|(?:"(.*?)")|(?:`(.*?)`))(?:\)\n*\s*|,)/g;
    TranslationParser.TRANSLATION_TEXT_REG_EXP = /^\s*(\$T|translate|core\.T)/;
    return TranslationParser;
}());
exports.default = TranslationParser;
//# sourceMappingURL=TranslationParser.js.map